package com.jpattern.orm.query.clause;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import com.jpattern.orm.query.BaseFindQuery;
import com.jpattern.orm.query.INameSolver;
import com.jpattern.orm.query.IQueryRoot;
import com.jpattern.orm.query.NullNameSolver;
import com.jpattern.orm.query.where.AndExpressionElement;
import com.jpattern.orm.query.where.EqExpressionElement;
import com.jpattern.orm.query.where.EqPropertiesExpressionElement;
import com.jpattern.orm.query.where.ExpressionElement;
import com.jpattern.orm.query.where.GeExpressionElement;
import com.jpattern.orm.query.where.GePropertiesExpressionElement;
import com.jpattern.orm.query.where.GtExpressionElement;
import com.jpattern.orm.query.where.GtPropertiesExpressionElement;
import com.jpattern.orm.query.where.IEqExpressionElement;
import com.jpattern.orm.query.where.IEqPropertiesExpressionElement;
import com.jpattern.orm.query.where.ILikeExpressionElement;
import com.jpattern.orm.query.where.InExpressionElement;
import com.jpattern.orm.query.where.InSubQueryExpressionElement;
import com.jpattern.orm.query.where.IsNotNullExpressionElement;
import com.jpattern.orm.query.where.IsNullExpressionElement;
import com.jpattern.orm.query.where.LeExpressionElement;
import com.jpattern.orm.query.where.LePropertiesExpressionElement;
import com.jpattern.orm.query.where.LikeExpressionElement;
import com.jpattern.orm.query.where.LtExpressionElement;
import com.jpattern.orm.query.where.LtPropertiesExpressionElement;
import com.jpattern.orm.query.where.NInExpressionElement;
import com.jpattern.orm.query.where.NLikeExpressionElement;
import com.jpattern.orm.query.where.NeExpressionElement;
import com.jpattern.orm.query.where.NePropertiesExpressionElement;
import com.jpattern.orm.query.where.NotExpressionElement;
import com.jpattern.orm.query.where.OrExpressionElement;

/**
 * 
 * @author Francesco Cina
 *
 * 19/giu/2011
 */
public class OrmWhere<T extends IQueryRoot> implements Where<T> {

	List<ExpressionElement> elementList = new ArrayList<ExpressionElement>();
	private INameSolver nameSolver = new NullNameSolver();
	private final T query;

	public OrmWhere(T query) {
		this.query = query;
	}

	@Override
	public Where<T> allEq(Map<String, Object> propertyMap) {
		for (final Entry<String, Object> entry : propertyMap.entrySet()) {
			eq(entry.getKey(), entry.getValue());
		}
		return this;
	}

	@Override
	public Where<T> eq(String property, Object value) {
		final ExpressionElement expressionElement = new EqExpressionElement(property, value);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> eqProperties(String firstProperty, String secondProperty) {
		final ExpressionElement expressionElement = new EqPropertiesExpressionElement(firstProperty, secondProperty);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> le(String property, Object value) {
		final ExpressionElement expressionElement = new LeExpressionElement(property, value);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> leProperties(String firstProperty, String secondProperty) {
		final ExpressionElement expressionElement = new LePropertiesExpressionElement(firstProperty, secondProperty);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> ge(String property, Object value) {
		final ExpressionElement expressionElement = new GeExpressionElement(property, value);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> geProperties(String firstProperty, String secondProperty) {
		final ExpressionElement expressionElement = new GePropertiesExpressionElement(firstProperty, secondProperty);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> lt(String property, Object value) {
		final ExpressionElement expressionElement = new LtExpressionElement(property, value);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> ltProperties(String firstProperty, String secondProperty) {
		final ExpressionElement expressionElement = new LtPropertiesExpressionElement(firstProperty, secondProperty);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> gt(String property, Object value) {
		final ExpressionElement expressionElement = new GtExpressionElement(property, value);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> gtProperties(String firstProperty, String secondProperty) {
		final ExpressionElement expressionElement = new GtPropertiesExpressionElement(firstProperty, secondProperty);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> ieq(String property, String value) {
		final ExpressionElement expressionElement = new IEqExpressionElement(property, value);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> ieqProperties(String firstProperty, String secondProperty) {
		final ExpressionElement expressionElement = new IEqPropertiesExpressionElement(firstProperty, secondProperty);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> ilike(String property, String value) {
		final ExpressionElement expressionElement = new ILikeExpressionElement(property, value);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> in(String property, Collection<?> values) {
		final ExpressionElement expressionElement = new InExpressionElement(property, values);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> in(String property, Object[] values) {
		return in(property, Arrays.asList( values ));
	}

	@Override
	public Where<T> in(String property, BaseFindQuery subQuery) {
		final ExpressionElement expressionElement = new InSubQueryExpressionElement(property, subQuery, true);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> nin(String property, Collection<?> values) {
		final ExpressionElement expressionElement = new NInExpressionElement(property, values);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> nin(String property, Object[] values) {
		return nin(property, Arrays.asList( values ));
	}

	@Override
	public Where<T> nin(String property, BaseFindQuery subQuery) {
		final ExpressionElement expressionElement = new InSubQueryExpressionElement(property, subQuery, false);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> isNotNull(String property) {
		final ExpressionElement expressionElement = new IsNotNullExpressionElement(property);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> isNull(String property) {
		final ExpressionElement expressionElement = new IsNullExpressionElement(property);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> like(String property, String value) {
		final ExpressionElement expressionElement = new LikeExpressionElement(property, value);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> nlike(String property, String value) {
		final ExpressionElement expressionElement = new NLikeExpressionElement(property, value);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> ne(String property, Object value) {
		final ExpressionElement expressionElement = new NeExpressionElement(property, value);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> neProperties(String firstProperty, String secondProperty) {
		final ExpressionElement expressionElement = new NePropertiesExpressionElement(firstProperty, secondProperty);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> not(ExpressionElement expression) {
		final ExpressionElement expressionElement = new NotExpressionElement(expression);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> or(ExpressionElement expressionOne, ExpressionElement expressionTwo) {
		final ExpressionElement expressionElement = new OrExpressionElement(expressionOne, expressionTwo);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public Where<T> and(ExpressionElement expressionOne, ExpressionElement expressionTwo) {
		final ExpressionElement expressionElement = new AndExpressionElement(expressionOne, expressionTwo);
		expressionElement.setNameSolver(nameSolver);
		elementList.add(expressionElement);
		return this;
	}

	@Override
	public void setNameSolver(INameSolver nameSolver) {
		this.nameSolver = nameSolver;
	}

	@Override
	public String renderSql() {
		final StringBuilder StringBuilder = new StringBuilder();
		renderSql(StringBuilder);
		return StringBuilder.toString();
	}

	@Override
	public void renderSql(StringBuilder StringBuilder) {
		boolean first = true;
		if (!elementList.isEmpty()) {
			StringBuilder.append("WHERE ");
			for (final ExpressionElement expressionElement : elementList) {
				if (!first) {
					StringBuilder.append("AND ");
				}
				expressionElement.renderSql(StringBuilder);
				first = false;
			}
		}
	}

	@Override
	public void appendValues(List<Object> values) {
		if (!elementList.isEmpty()) {
			for (final ExpressionElement expressionElement : elementList) {
				expressionElement.appendValues(values);
			}
		}
	}

	@Override
	public T query() {
		return query;
	}

}
